import { Multiaddr, Resolver } from '@multiformats/multiaddr';
import type { Connection } from '@libp2p/interface-connection';
import type { AbortOptions } from '@libp2p/interfaces';
import type { PeerId } from '@libp2p/interface-peer-id';
import type { AddressSorter, PeerStore } from '@libp2p/interface-peer-store';
import type { Metrics } from '@libp2p/interface-metrics';
import type { TransportManager } from '@libp2p/interface-transport';
import type { ConnectionGater } from '@libp2p/interface-connection-gater';
import PQueue from 'p-queue';
export type PendingDialStatus = 'queued' | 'active' | 'error' | 'success';
export interface PendingDial {
    id: string;
    status: PendingDialStatus;
    peerId?: PeerId;
    multiaddrs: Multiaddr[];
    promise: Promise<Connection>;
}
export interface PendingDialTarget {
    resolve: (value: any) => void;
    reject: (err: Error) => void;
}
export interface DialOptions extends AbortOptions {
    priority?: number;
}
interface DialerInit {
    addressSorter?: AddressSorter;
    maxParallelDials?: number;
    maxPeerAddrsToDial?: number;
    maxParallelDialsPerPeer?: number;
    dialTimeout?: number;
    resolvers?: Record<string, Resolver>;
}
interface DialQueueComponents {
    peerId: PeerId;
    metrics?: Metrics;
    peerStore: PeerStore;
    transportManager: TransportManager;
    connectionGater: ConnectionGater;
}
export declare class DialQueue {
    pendingDials: PendingDial[];
    queue: PQueue;
    private readonly peerId;
    private readonly peerStore;
    private readonly connectionGater;
    private readonly transportManager;
    private readonly addressSorter;
    private readonly maxPeerAddrsToDial;
    private readonly maxParallelDialsPerPeer;
    private readonly dialTimeout;
    private readonly inProgressDialCount?;
    private readonly pendingDialCount?;
    private readonly shutDownController;
    constructor(components: DialQueueComponents, init?: DialerInit);
    /**
     * Clears any pending dials
     */
    stop(): void;
    /**
     * Connects to a given peer, multiaddr or list of multiaddrs.
     *
     * If a peer is passed, all known multiaddrs will be tried. If a multiaddr or
     * multiaddrs are passed only those will be dialled.
     *
     * Where a list of multiaddrs is passed, if any contain a peer id then all
     * multiaddrs in the list must contain the same peer id.
     *
     * The dial to the first address that is successfully able to upgrade a connection
     * will be used, all other dials will be aborted when that happens.
     */
    dial(peerIdOrMultiaddr: PeerId | Multiaddr | Multiaddr[], options?: DialOptions): Promise<Connection>;
    private createDialAbortControllers;
    private calculateMultiaddrs;
    private performDial;
}
export {};
//# sourceMappingURL=dial-queue.d.ts.map